@@ -20,6 +20,7 @@
  */
 
 #include <glib/gi18n.h>
+#include <stdlib.h>
 #include <vinagre/vinagre-utils.h>
 #include "vinagre-vnc-connection.h"
 
@@ -29,6 +30,10 @@ struct _VinagreVncConnectionPrivate
   gboolean view_only;
   gboolean scaling;
   gint     shared;
+  gint     fd;
+  gint     depth_profile;
+  gboolean lossy_encoding;
+  gchar    *ssh_tunnel_host;
 };
 
 enum
@@ -38,6 +43,10 @@ enum
   PROP_VIEW_ONLY,
   PROP_SCALING,
   PROP_SHARED,
+  PROP_FD,
+  PROP_DEPTH_PROFILE,
+  PROP_LOSSY_ENCODING,
+  PROP_SSH_TUNNEL_HOST
 };
 
 #define VINAGRE_VNC_CONNECTION_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VINAGRE_TYPE_VNC_CONNECTION, VinagreVncConnectionPrivate))
@@ -52,6 +61,10 @@ vinagre_vnc_connection_init (VinagreVncConnection *conn)
   conn->priv->view_only = FALSE;
   conn->priv->scaling = FALSE;
   conn->priv->shared = -1;
+  conn->priv->fd = 0;
+  conn->priv->depth_profile = 0;
+  conn->priv->lossy_encoding = FALSE;
+  conn->priv->ssh_tunnel_host = NULL;
 }
 
 static void
@@ -66,6 +79,7 @@ vinagre_vnc_connection_finalize (GObject *object)
   VinagreVncConnection *conn = VINAGRE_VNC_CONNECTION (object);
 
   g_free (conn->priv->desktop_name);
+  g_free (conn->priv->ssh_tunnel_host);
 
   G_OBJECT_CLASS (vinagre_vnc_connection_parent_class)->finalize (object);
 }
@@ -97,6 +111,22 @@ vinagre_vnc_connection_set_property (GObject *object, guint prop_id, const GValu
 	vinagre_vnc_connection_set_shared (conn, g_value_get_int (value));
 	break;
 
+      case PROP_FD:
+	vinagre_vnc_connection_set_fd (conn, g_value_get_int (value));
+	break;
+
+      case PROP_DEPTH_PROFILE:
+	vinagre_vnc_connection_set_depth_profile (conn, g_value_get_int (value));
+	break;
+
+      case PROP_LOSSY_ENCODING:
+	vinagre_vnc_connection_set_lossy_encoding (conn, g_value_get_boolean (value));
+	break;
+
+      case PROP_SSH_TUNNEL_HOST:
+	vinagre_vnc_connection_set_ssh_tunnel_host (conn, g_value_get_string (value));
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -130,6 +160,22 @@ vinagre_vnc_connection_get_property (GObject *object, guint prop_id, GValue *val
 	g_value_set_int (value, conn->priv->shared);
 	break;
 
+      case PROP_FD:
+	g_value_set_int (value, conn->priv->fd);
+	break;
+
+      case PROP_DEPTH_PROFILE:
+	g_value_set_int (value, conn->priv->depth_profile);
+	break;
+
+      case PROP_LOSSY_ENCODING:
+	g_value_set_boolean (value, conn->priv->lossy_encoding);
+	break;
+
+      case PROP_SSH_TUNNEL_HOST:
+	g_value_set_string (value, conn->priv->ssh_tunnel_host);
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -144,6 +190,11 @@ vnc_fill_writer (VinagreConnection *conn, xmlTextWriter *writer)
 
   xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"view_only", "%d", vnc_conn->priv->view_only);
   xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"scaling", "%d", vnc_conn->priv->scaling);
+  xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"depth_profile", "%d", vnc_conn->priv->depth_profile);
+  xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"lossy_encoding", "%d", vnc_conn->priv->lossy_encoding);
+
+  if (vnc_conn->priv->ssh_tunnel_host && *vnc_conn->priv->ssh_tunnel_host)
+    xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"ssh_tunnel_host", "%s", vnc_conn->priv->ssh_tunnel_host);
 }
 
 static void
@@ -160,12 +211,26 @@ vnc_parse_item (VinagreConnection *conn, xmlNode *root)
       s_value = xmlNodeGetContent (curr);
 
       if (!xmlStrcmp(curr->name, (const xmlChar *)"view_only"))
-	vinagre_vnc_connection_set_view_only (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
+	{
+	  vinagre_vnc_connection_set_view_only (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
+	}
       else if (!xmlStrcmp(curr->name, (const xmlChar *)"scaling"))
 	{
 	  if (!scaling_command_line)
 	    vinagre_vnc_connection_set_scaling (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
 	}
+      else if (!xmlStrcmp(curr->name, (const xmlChar *)"depth_profile"))
+	{
+	  vinagre_vnc_connection_set_depth_profile (vnc_conn, atoi((const char *)s_value));
+	}
+      else if (!xmlStrcmp(curr->name, (const xmlChar *)"lossy_encoding"))
+	{
+	  vinagre_vnc_connection_set_lossy_encoding (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
+	}
+      else if (!xmlStrcmp(curr->name, (const xmlChar *)"ssh_tunnel_host"))
+	{
+	  vinagre_vnc_connection_set_ssh_tunnel_host (vnc_conn, (const gchar *)s_value);
+	}
 
       xmlFree (s_value);
     }
@@ -210,11 +275,14 @@ vnc_fill_conn_from_file (VinagreConnection *conn, GKeyFile *file)
 static void
 vnc_parse_options_widget (VinagreConnection *conn, GtkWidget *widget)
 {
-  GtkWidget *view_only, *scaling;
+  GtkWidget *view_only, *scaling, *depth_combo, *lossy, *ssh_host;
 
   view_only = g_object_get_data (G_OBJECT (widget), "view_only");
   scaling = g_object_get_data (G_OBJECT (widget), "scaling");
-  if (!view_only || !scaling)
+  depth_combo = g_object_get_data (G_OBJECT (widget), "depth_combo");
+  lossy = g_object_get_data (G_OBJECT (widget), "lossy");
+  ssh_host = g_object_get_data (G_OBJECT (widget), "ssh_host");
+  if (!view_only || !scaling || !depth_combo || !lossy || !ssh_host)
     {
       g_warning ("Wrong widget passed to vnc_parse_options_widget()");
       return;
@@ -223,6 +291,9 @@ vnc_parse_options_widget (VinagreConnection *conn, GtkWidget *widget)
   g_object_set (conn,
 		"view-only", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view_only)),
 		"scaling", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (scaling)),
+		"depth-profile", gtk_combo_box_get_active (GTK_COMBO_BOX (depth_combo)),
+		"lossy-encoding", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lossy)),
+		"ssh-tunnel-host", gtk_entry_get_text (GTK_ENTRY (ssh_host)),
 		NULL);
 }
 
@@ -294,6 +365,58 @@ vinagre_vnc_connection_class_init (VinagreVncConnectionClass *klass)
                                                       G_PARAM_STATIC_NAME |
                                                       G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (object_class,
+                                   PROP_FD,
+                                   g_param_spec_int ("fd",
+                                                     "file descriptor",
+	                                              "the file descriptor for this connection",
+                                                      0,
+                                                      G_MAXINT,
+                                                      0,
+	                                              G_PARAM_READWRITE |
+                                                      G_PARAM_CONSTRUCT |
+                                                      G_PARAM_STATIC_NICK |
+                                                      G_PARAM_STATIC_NAME |
+                                                      G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+                                   PROP_DEPTH_PROFILE,
+                                   g_param_spec_int ("depth-profile",
+                                                     "Depth Profile",
+	                                              "The profile of depth color to be used in gtk-vnc widget",
+                                                      0,
+                                                      5,
+                                                      0,
+	                                              G_PARAM_READWRITE |
+                                                      G_PARAM_CONSTRUCT |
+                                                      G_PARAM_STATIC_NICK |
+                                                      G_PARAM_STATIC_NAME |
+                                                      G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+                                   PROP_LOSSY_ENCODING,
+                                   g_param_spec_boolean ("lossy-encoding",
+                                                        "Lossy encoding",
+	                                                "Whether to use a lossy encoding",
+                                                        FALSE,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+                                   PROP_SSH_TUNNEL_HOST,
+                                   g_param_spec_string ("ssh-tunnel-host",
+                                                        "SSH Tunnel Host",
+	                                                "hostname used to create the SSH tunnel",
+                                                        NULL,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+
 }
 
 VinagreConnection *
@@ -368,4 +491,71 @@ vinagre_vnc_connection_get_scaling (VinagreVncConnection *conn)
   return conn->priv->scaling;
 }
 
+void
+vinagre_vnc_connection_set_fd (VinagreVncConnection *conn,
+			       gint                 value)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+  g_return_if_fail (value >= 0);
+
+  conn->priv->fd = value;
+}
+gint
+vinagre_vnc_connection_get_fd (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), 0);
+
+  return conn->priv->fd;
+}
+
+void
+vinagre_vnc_connection_set_depth_profile (VinagreVncConnection *conn,
+					  gint                 value)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+  g_return_if_fail (value >= 0);
+
+  conn->priv->depth_profile = value;
+}
+gint
+vinagre_vnc_connection_get_depth_profile (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), 0);
+
+  return conn->priv->depth_profile;
+}
+
+void
+vinagre_vnc_connection_set_lossy_encoding (VinagreVncConnection *conn,
+					   gboolean value)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+
+  conn->priv->lossy_encoding = value;
+}
+gboolean
+vinagre_vnc_connection_get_lossy_encoding (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), FALSE);
+
+  return conn->priv->lossy_encoding;
+}
+
+void
+vinagre_vnc_connection_set_ssh_tunnel_host (VinagreVncConnection *conn,
+					    const gchar *host)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+
+  g_free (conn->priv->ssh_tunnel_host);
+  conn->priv->ssh_tunnel_host = g_strdup (host);
+}
+const gchar *
+vinagre_vnc_connection_get_ssh_tunnel_host (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), NULL);
+
+  return conn->priv->ssh_tunnel_host;
+}
+
 /* vim: set ts=8: */
