@@ -191,7 +191,7 @@ saved_history (void)
   gint len;
   GPtrArray *array;
 
-  array = g_ptr_array_new ();
+  array = g_ptr_array_new_with_free_func (g_free);
 
   filename = history_filename ();
   success = g_file_get_contents (filename,
@@ -217,6 +217,7 @@ saved_history (void)
 
   g_free (filename);
   g_free (file_contents);
+  g_free (history_from_file);
   return array;
 }
 
@@ -379,6 +380,8 @@ VinagreConnection *vinagre_connect (VinagreWindow *window)
   VinagreConnectDialog  dialog;
 
   dialog.xml = vinagre_utils_get_builder (NULL, NULL);
+  if (!dialog.xml)
+    return NULL;
 
   dialog.dialog = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "connect_dialog"));
   gtk_window_set_transient_for (GTK_WINDOW (dialog.dialog), GTK_WINDOW (window));
